"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function AccountBalanceMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AccountBalanceMSSql = AccountBalanceMSSql;
function CreateMainAccountBalance(plataformHash, accountId, balance, user) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), plataformHash)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('Balance', sql.Decimal(18, 5), balance)
        .input('User', sql.VarChar(250), user)
        .execute('CreateMainAccountBalance')
        .then(function (recordsets) {
        logger.info('[AccountBalanceMSSql.mssql::CreateMainAccountBalance] Result: ', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.CreateMainAccountBalance = CreateMainAccountBalance;
function GetAccountSubAccountRate(installationId, accountId, subAccountId, searchType) {
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('SubAccountId', sql.VarChar(50), subAccountId)
        .input('SearchType', sql.VarChar(50), searchType)
        .execute('GetAccountSubAccountRate')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAccountSubAccountRate = GetAccountSubAccountRate;
function GetAccountSubAccountBalance(installationId, accountId, subAccountId, systemService, searchType) {
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    if (systemService === void 0) { systemService = null; }
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('SubAccountId', sql.VarChar(50), subAccountId)
        .input('SystemService', sql.VarChar(50), systemService)
        .input('SearchType', sql.VarChar(50), searchType)
        .execute('GetAccountSubAccountBalance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAccountSubAccountBalance = GetAccountSubAccountBalance;
function GetSubAccountsBalance(installationId, accountId) {
    if (accountId === void 0) { accountId = null; }
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .execute('GetSubAccountsBalance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSubAccountsBalance = GetSubAccountsBalance;
function GetVCCBalance(installationId, accountId) {
    if (accountId === void 0) { accountId = null; }
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .execute('GetVCCBalance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetVCCBalance = GetVCCBalance;
function GetAccountSubAccountVCCRetainedBalance(installationId, accountId, subAccountId, vcc) {
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    if (vcc === void 0) { vcc = null; }
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('SubAccountId', sql.VarChar(50), subAccountId)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetAccountSubAccountVCCRetainedBalance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAccountSubAccountVCCRetainedBalance = GetAccountSubAccountVCCRetainedBalance;
