"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var guid_module_1 = require("../../shared/guid.module");
var logger = log4js.getLogger('Sql');
var sqlConn;
function AccountVccBalanceMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AccountVccBalanceMSSql = AccountVccBalanceMSSql;
function GetAccountSubAccountVCCBalance(platformHash, installationId, accountId, subAccountId, vcc, systemService) {
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    if (vcc === void 0) { vcc = null; }
    if (systemService === void 0) { systemService = null; }
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('SubAccountId', sql.VarChar(50), subAccountId)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SystemService', sql.VarChar(50), systemService)
        .execute('GetAccountSubAccountVCCBalance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAccountSubAccountVCCBalance = GetAccountSubAccountVCCBalance;
function GetAccountSubAccountVCCBalanceSummary(platformHash, installationId, accountId, subAccountId, vcc, systemService) {
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    if (vcc === void 0) { vcc = null; }
    if (systemService === void 0) { systemService = null; }
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('SubAccountId', sql.VarChar(50), subAccountId)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SystemService', sql.VarChar(50), systemService)
        .execute('GetAccountSubAccountVCCBalanceSummary')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAccountSubAccountVCCBalanceSummary = GetAccountSubAccountVCCBalanceSummary;
function AccountSubAccountVCCBalanceAllocation(id, step, platformHash, installationId, accountId, subAccountId, vcc, balance, systemService, operation, user) {
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    if (vcc === void 0) { vcc = null; }
    if (balance === void 0) { balance = 0; }
    if (systemService === void 0) { systemService = null; }
    if (operation === void 0) { operation = null; }
    if (user === void 0) { user = null; }
    return new sql.Request(sqlConn)
        .input('Id', sql.VarChar(100), id)
        .input('Step', sql.Int, step)
        .input('PlatformHash', sql.VarChar(50), platformHash)
        .input('InstallationId', sql.VarChar(50), installationId)
        .input('AccountId', sql.VarChar(50), accountId)
        .input('SubAccountId', sql.VarChar(50), subAccountId)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Balance', sql.Decimal(18, 5), balance)
        .input('Target', sql.VarChar(50), systemService)
        .input('Operation', sql.VarChar(50), operation)
        .input('User', sql.VarChar(50), user)
        .execute('AccountSubAccountVCCBalanceAllocation')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.AccountSubAccountVCCBalanceAllocation = AccountSubAccountVCCBalanceAllocation;
function TransactionAccountSubAccountVCCBalanceAllocation(platformHash, installationId, accountId, subAccountId, vcc, data, user) {
    var _this = this;
    if (platformHash === void 0) { platformHash = null; }
    if (installationId === void 0) { installationId = null; }
    if (accountId === void 0) { accountId = null; }
    if (subAccountId === void 0) { subAccountId = null; }
    if (vcc === void 0) { vcc = null; }
    if (data === void 0) { data = []; }
    if (user === void 0) { user = null; }
    var transaction = new sql.Transaction(sqlConn);
    var movementId = guid_module_1.GenerateGuid();
    var step = 0;
    return transaction.begin()
        .then(function (result) {
        return new sql.Request(transaction);
    })
        .then(function (result) {
        var myDecreaseServices = data.filter(function (e) { return e.operation === 'decrease' && e.target !== 'free'; });
        var promises = [];
        myDecreaseServices.forEach(function (a) {
            promises.push(_this.AccountSubAccountVCCBalanceAllocation(movementId, step, platformHash, installationId, accountId, subAccountId, vcc, a.balance, a.target.toUpperCase(), a.operation.toUpperCase(), user));
            step++;
        });
        return Promise.all(promises);
    })
        .then(function (result) {
        var myIncreaseServices = data.filter(function (e) { return e.operation === 'increase' && e.target !== 'free'; });
        var promises = [];
        myIncreaseServices.forEach(function (a) {
            promises.push(_this.AccountSubAccountVCCBalanceAllocation(movementId, step, platformHash, installationId, accountId, subAccountId, vcc, a.balance, a.target.toUpperCase(), a.operation.toUpperCase(), user));
            step++;
        });
        return Promise.all(promises);
    })
        .then(function (result) {
        return transaction.commit();
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        transaction.rollback();
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.TransactionAccountSubAccountVCCBalanceAllocation = TransactionAccountSubAccountVCCBalanceAllocation;
