"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function PlatformMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.PlatformMSSql = PlatformMSSql;
function GetPlatforms(search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetPlatforms')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetPlatforms = GetPlatforms;
function GetPlatform(platformHash) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .execute('GetPlatform')
        .then(function (recordsets) {
        return Promise.resolve(recordsets);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetPlatform = GetPlatform;
function GetPlatformByHash(platformHash) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .execute('GetPlatformByHash')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var platform = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(platform);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetPlatformByHash = GetPlatformByHash;
function GetPlatformByData(installationId, macAddress, publicIp) {
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(50), installationId)
        .input('MacAddress', sql.VarChar(50), macAddress)
        .input('PublicIp', sql.VarChar(50), publicIp)
        .execute('GetPlatformByData')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var platform = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(platform);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetPlatformByData = GetPlatformByData;
function SavePlatformHash(platformHash, type, environmentId, user) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .input('Type', sql.VarChar(25), type)
        .input('EnvironmentId', sql.VarChar(50), environmentId)
        .input('User', sql.VarChar(50), user)
        .execute('SavePlatformHash')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SavePlatformHash = SavePlatformHash;
function SavePlatform(platformHash, type, environmentId) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .input('Type', sql.VarChar(50), type)
        .input('EnvironmentId', sql.VarChar(100), environmentId)
        .execute('SavePlatform')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var platform = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(platform);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SavePlatform = SavePlatform;
function ActivatePlatform(platformHash, installationId, macAddress, publicIp, user, token) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .input('InstallationId', sql.VarChar(50), installationId)
        .input('MacAddress', sql.VarChar(50), macAddress)
        .input('PublicIp', sql.VarChar(50), publicIp)
        .input('User', sql.VarChar(50), user)
        .input('Token', sql.VarChar(50), token)
        .execute('ActivatePlatform')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var platform = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(platform);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.ActivatePlatform = ActivatePlatform;
function UpdatePlatformHash(platformHash, platformHashNew, user) {
    return new sql.Request(sqlConn)
        .input('PlatformHash', sql.VarChar(100), platformHash)
        .input('PlatformHashNew', sql.VarChar(100), platformHashNew)
        .input('User', sql.VarChar(50), user)
        .execute('UpdatePlatformHash')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdatePlatformHash = UpdatePlatformHash;
function GetInstallationIds(installationId) {
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .execute('GetInstallationIds')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInstallationIds = GetInstallationIds;
