"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var redis = require("redis");
var bluebird = require("bluebird");
var log4js = require("log4js");
var redis_config_1 = require("../config/redis.config");
var domain_redis_1 = require("./redis/domain.redis");
var vcc_redis_1 = require("./redis/vcc.redis");
var role_redis_1 = require("./redis/role.redis");
var session_redis_1 = require("./redis/session.redis");
var user_redis_1 = require("./redis/user.redis");
var queue_redis_1 = require("./redis/queue.redis");
var logger = log4js.getLogger('Redis');
function RedisInit(app) {
    return new Promise(function (resolve, reject) {
        logger.info('Iniciando conexión a Redis');
        console.log('Iniciando conexión a Redis');
        var redisClient = redis.createClient(redis_config_1.REDIS_OPTIONS)
            .on('error', function (err) {
            var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('No se pudo inicializar cliente Redis: ' + errMsg);
            console.error('No se pudo inicializar cliente Redis: ' + errMsg);
            reject(err);
        })
            .on('connect', function () {
            logger.info('Conexión a Redis establecida');
            console.log('Conexión a Redis establecida');
            resolve(true);
        });
        bluebird.promisifyAll(redisClient);
        app.set('redisClient', redisClient);
        domain_redis_1.DomainRedis(app);
        vcc_redis_1.VccRedis(app);
        role_redis_1.RoleRedis(app);
        session_redis_1.SessionRedis(app);
        user_redis_1.UserRedis(app);
        queue_redis_1.QueueRedis(app);
    });
}
exports.RedisInit = RedisInit;
