"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var platform_model_1 = require("../models/platform.model");
var logger = log4js.getLogger('ServerScripts');
function AccountService(app) {
    app.get('/api/account/get_all_accounts/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AccountService::get_all_accounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAllAccounts(search_criteria, search_expression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_all_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/account/get_accounts/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AccountService::get_accounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAccounts(search_criteria, search_expression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var id = req.body.id || null;
        var accountId = req.body.account || null;
        var subAccountId = req.body.subAccountId || null;
        var platform = new platform_model_1.Platform;
        var resData = {
            account: null,
            accountBalance: null,
            platform: null,
            vcc: null
        };
        logger.info('[AccountService::get_account1] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (platformHash && id) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[AccountService::get_account] Received params: ', platformHash, id);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_PLATFORM_NOT_FOUND';
            else if (res.headersSent)
                return;
            platform = result;
            resData.platform = platform;
            return mssql_service_1.MSSql.GetAccount(platformHash, platform.installationId, id);
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_NOT_FOUND';
            else {
                resData.account = result;
                if (result.subAccountId)
                    return mssql_service_1.MSSql.GetAccountSubAccountBalance(platform.installationId, result.accountId, result.subAccountId, null, 'SCTA');
                else
                    return mssql_service_1.MSSql.GetAccountSubAccountBalance(platform.installationId, result.accountId, null, null, 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_BALANCE';
            else {
                resData.accountBalance = result;
                if (resData.account.subAccountId) {
                    return Promise.resolve([]);
                }
                else {
                    return mssql_service_1.MSSql.GetAccountSubAccountVCCRetainedBalance(platform.installationId, resData.account.accountId, null, null);
                }
            }
        }).then(function (result) {
            if (res.headersSent)
                return;
            resData.vcc = result;
            res.json({ status: true, description: 'OK', data: resData });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_account] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_accounts', function (req, res, next) {
        var installationId = req.body.installationId || null;
        var updated = req.body.updated || false;
        logger.info('[AccountService::get_accounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetRelatedAccounts(installationId);
            }
            else {
                logger.info('[AccountService::get_accounts] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_subaccounts', function (req, res, next) {
        var installationId = req.body.installationId || null;
        logger.info('[AccountService::get_subaccounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetRelatedSubAccounts(installationId);
            }
            else {
                logger.info('[AccountService::get_subaccounts] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_subaccounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account_balance', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var accountId = req.body.account || null;
        var resData = {
            accountBalance: null,
            accountRate: null,
            subAccountsBalance: null,
            subAccountsRate: null
        };
        logger.info('[AccountService::get_account_balance] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId && accountId) {
                return mssql_service_1.MSSql.GetAccountSubAccountExists(installationId, accountId, null);
            }
            else {
                logger.info('[AccountService::get_account_balance] Received params: ', installationId, accountId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_NOT_FOUND';
            else {
                if (res.headersSent)
                    return;
                return mssql_service_1.MSSql.GetAccountSubAccountRate(installationId, accountId, null, 'CTA');
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_RATE';
            else {
                if (res.headersSent)
                    return;
                resData.accountRate = result[0];
                return mssql_service_1.MSSql.GetAccountSubAccountBalance(installationId, accountId, null, 'FREE', 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_BALANCE';
            else {
                if (res.headersSent)
                    return;
                resData.accountBalance = result[0];
                return mssql_service_1.MSSql.GetAccountSubAccountRate(installationId, accountId, null, 'SCTA');
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT_RATES';
            else {
                if (res.headersSent)
                    return;
                resData.subAccountsRate = result;
                return mssql_service_1.MSSql.GetAccountSubAccountBalance(installationId, accountId, null, 'FREE', 'SCTA');
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT_BALANCES';
            else {
                if (res.headersSent)
                    return;
                resData.subAccountsBalance = result;
                res.json({ status: true, description: 'OK', data: resData });
            }
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_account_balance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_subaccounts_vcc', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var accountId = req.body.account || null;
        var resData = {
            subAccountBalance: [],
            vccBalance: []
        };
        logger.info('[AccountService::get_subaccounts_vcc] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId && accountId) {
                return mssql_service_1.MSSql.GetSubAccountsBalance(installationId, accountId);
            }
            else {
                logger.info('[AccountService::get_subaccounts_vcc Received params: ', installationId, accountId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (typeof result != 'string') {
                resData.subAccountBalance = result;
            }
            return mssql_service_1.MSSql.GetVCCBalance(installationId, accountId);
        })
            .then(function (result) {
            if (typeof result != 'string') {
                resData.vccBalance = result;
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resData });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_subaccounts_vcc] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/save_account', function (req, res, next) {
        var id = req.body.id || null;
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var accountId = req.body.accountId || null;
        var subAccountId = req.body.subAccountId || null;
        var currencyCode = req.body.currencyCode || null;
        var user = req.body.user || null;
        var token = req.body.token || null;
        var initialBalance = req.body.initialBalance || 0;
        var transcriptionRatePerHour = req.body.transcriptionRatePerHour || 0;
        var semanticAnalysisRatePerTransaction = req.body.semanticAnalysisRatePerTransaction || 0;
        var cognitiveAnalysisRatePerTransaction = req.body.cognitiveAnalysisRatePerTransaction || 0;
        var myUser = res.locals.userId;
        logger.info('[AccountService::save_account] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (0 < transcriptionRatePerHour
                &&
                    transcriptionRatePerHour < 0.036)
                throw 'SERVER_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION';
            if (platformHash && installationId && accountId && user && token
                && !isNaN(transcriptionRatePerHour)
                && !isNaN(semanticAnalysisRatePerTransaction)
                && !isNaN(cognitiveAnalysisRatePerTransaction)) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[PlatformService::save_account] Received params: ', platformHash, installationId, accountId, user, token, transcriptionRatePerHour, semanticAnalysisRatePerTransaction, cognitiveAnalysisRatePerTransaction);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_PLATFORM_NOT_FOUND';
            else
                return mssql_service_1.MSSql.SaveAccount(platformHash, id, installationId, accountId, subAccountId, currencyCode, user, token, transcriptionRatePerHour, (transcriptionRatePerHour / 3600), semanticAnalysisRatePerTransaction, cognitiveAnalysisRatePerTransaction, myUser, initialBalance);
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_SAVING_ERROR';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/activate_account', function (req, res, next) {
        var id = req.body.id || null;
        var accountId = req.body.accountId || null;
        var subAccountId = req.body.subAccountId || null;
        var installationId = req.body.installationId || null;
        var myUser = res.locals.userId;
        logger.info('[AccountService::activate_account] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (id && accountId && installationId) {
                return mssql_service_1.MSSql.AccountSubAccountActivate(installationId, accountId, subAccountId, myUser);
            }
            else {
                logger.info('[PlatformService::activate_account] Received params: ', id, accountId, subAccountId, installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::activate_account] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/deactivate_account', function (req, res, next) {
        var id = req.body.id || null;
        var accountId = req.body.accountId || null;
        var subAccountId = req.body.subAccountId || null;
        var installationId = req.body.installationId || null;
        var myUser = res.locals.userId;
        logger.info('[AccountService::deactivate_account] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (id && accountId && installationId) {
                return mssql_service_1.MSSql.AccountSubAccountDeactivate(installationId, accountId, subAccountId, myUser);
            }
            else {
                logger.info('[PlatformService::deactivate_account] Received params: ', id, accountId, subAccountId, installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::deactivate_account] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_vccs_by_account', function (req, res, next) {
        var installationId = req.body.installationId || null;
        var accountId = req.body.account || null;
        var type = req.body.type || null;
        logger.info('[AccountService::get_vccs_by_account] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId && accountId) {
                return mssql_service_1.MSSql.GetVCCsByAccount(installationId, accountId, type);
            }
            else {
                logger.info('[AccountService::get_vccs_by_account] Received params: ', installationId, accountId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_vccs_by_account] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_movements', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var startDate = new Date(req.body.startDate) || null;
        var endDate = new Date(req.body.endDate) || null;
        var accountId = req.body.accountId || null;
        var subAccountId = req.body.subAccountId || null;
        var vcc = req.body.vcc || null;
        var currencies = req.body.currencies || [];
        var type = req.body.type || null;
        logger.info('[AccountService::/api/account/get_movements] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return GetMovements(installationId, startDate, endDate, accountId, subAccountId, vcc, currencies);
        }).then(function (result) {
            var data;
            if (result) {
                data = {
                    data: {
                        TRANSCRIPTIONS: result.filter(function (x) { return x.service === 'TRANSCRIPTION'; }),
                        SEMANTIC_ANALYSIS: result.filter(function (x) { return x.service === 'SEMANTIC_ANALYSIS'; }),
                        COGNITIVE_ANALYSIS: result.filter(function (x) { return x.service === 'COGNITIVE_ANALYSIS'; })
                    }
                };
            }
            else {
                data = {
                    data: {
                        TRANSCRIPTIONS: [],
                        SEMANTIC_ANALYSIS: [],
                        COGNITIVE_ANALYSIS: []
                    }
                };
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::/api/account/get_movements] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account_status_by_vcc', function (req, res, next) {
        var installationId = req.body.installationId || null;
        var vcc = req.body.vcc || null;
        logger.info('[AccountService::get_account_status_by_vcc] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetAccountStatusByVCC(installationId, vcc, 1);
            }
            else {
                logger.info('[AccountService::get_account_status_by_vcc] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_account_status_by_vcc] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/recharge', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var accountId = req.body.account || null;
        var subAccountId = req.body.subAccount || null;
        var vcc = req.body.vcc || null;
        var data = req.body.data || null;
        var myUser = res.locals.userId || null;
        var rechargeResult = [];
        logger.info('[AccountService::recharge] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (platformHash && installationId && accountId && data) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[AccountService::recharge] Received params: ', platformHash, installationId, accountId, data);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_PLATFORM_NOT_FOUND';
            else {
                var account = data.find(function (e) { return e.target === 'account'; });
                if (account && account.balance && account.balance != 0) {
                    return mssql_service_1.MSSql.RechargeAccountSubAccountVCC(platformHash, installationId, accountId, subAccountId, vcc, account.balance, account.target.toUpperCase(), myUser);
                }
                else {
                    Promise.resolve(true);
                }
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_UPDATE_ACCOUNT_BALANCE_FAILED';
            else {
                var account = data.find(function (e) { return e.target === 'account'; });
                if (account && account.balance && account.balance != 0)
                    rechargeResult.push('SERVER_RESULT_UPDATE_ACCOUNT_BALANCE_SUCCESS');
                var subaccount = data.find(function (e) { return e.target === 'subaccount'; });
                if (subaccount && subaccount.balance && subaccount.balance != 0) {
                    return mssql_service_1.MSSql.RechargeAccountSubAccountVCC(platformHash, installationId, accountId, subAccountId, vcc, subaccount.balance, subaccount.target.toUpperCase(), myUser);
                }
                else {
                    Promise.resolve(true);
                }
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_UPDATE_SUBACCOUNT_BALANCE_FAILED';
            else {
                var subaccount = data.find(function (e) { return e.target === 'subaccount'; });
                if (subaccount && subaccount.balance && subaccount.balance != 0) {
                    rechargeResult.push('SERVER_RESULT_UPDATE_SUBACCOUNT_BALANCE_SUCCESS');
                }
                var selectedVcc = data.find(function (e) { return e.target === 'vcc'; });
                if (selectedVcc && selectedVcc.balance && selectedVcc.balance != 0) {
                    return mssql_service_1.MSSql.RechargeAccountSubAccountVCC(platformHash, installationId, accountId, subAccountId, vcc, selectedVcc.balance, selectedVcc.target.toUpperCase(), myUser);
                }
                else {
                    Promise.resolve(true);
                }
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_UPDATE_BALANCE_FAILED';
            else {
                var selectedVcc = data.find(function (e) { return e.target === 'vcc'; });
                if (selectedVcc && selectedVcc.balance && selectedVcc.balance != 0) {
                    rechargeResult.push('SERVER_RESULT_UPDATE_ACCOUNT_VCC_SUCCESS');
                }
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: rechargeResult });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::recharge] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_costs', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var startDate = req.body.startDate || null;
        var endDate = req.body.endDate || null;
        var accountId = req.body.accountId || null;
        var subAccountId = req.body.subAccountId || null;
        var vcc = req.body.vcc || null;
        var type = req.body.type || null;
        logger.info('[AccountService::get_costs] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return GetCosts(installationId, startDate, endDate, accountId, subAccountId, vcc);
        }).then(function (result) {
            var response = [];
            if (result.length > 0) {
                if (result.length > 4) {
                    var cognitive = result.filter(function (x) { return x.name == 'COGNITIVE_ANALYSIS'; });
                    var semantic = result.filter(function (x) { return x.name == 'SEMANTIC_ANALYSIS'; });
                    var free = result.filter(function (x) { return x.name == 'SEMANTIC_ANALYSIS'; });
                    var transcription = result.filter(function (x) { return x.name == 'TRANSCRIPTION'; });
                    var amount_1 = 0, retained_1 = 0, available_1 = 0;
                    cognitive.forEach(function (x) {
                        amount_1 += x.amount;
                        retained_1 += x.retained;
                        available_1 += x.available;
                    });
                    response.push({ name: 'COGNITIVE_ANALYSIS', amount: amount_1, retained: retained_1, available: available_1 });
                    amount_1 = 0, retained_1 = 0, available_1 = 0;
                    semantic.forEach(function (x) {
                        amount_1 += x.amount;
                        retained_1 += x.retained;
                        available_1 += x.available;
                    });
                    response.push({ name: 'SEMANTIC_ANALYSIS', amount: amount_1, retained: retained_1, available: available_1 });
                    amount_1 = 0, retained_1 = 0, available_1 = 0;
                    transcription.forEach(function (x) {
                        amount_1 += x.amount;
                        retained_1 += x.retained;
                        available_1 += x.available;
                    });
                    response.push({ name: 'TRANSCRIPTION', amount: amount_1, retained: retained_1, available: available_1 });
                    amount_1 = 0, retained_1 = 0, available_1 = 0;
                    free.forEach(function (x) {
                        amount_1 += x.amount;
                        retained_1 += x.retained;
                        available_1 += x.available;
                    });
                    response.push({ name: 'FREE', amount: amount_1, retained: retained_1, available: available_1 });
                    amount_1 = 0, retained_1 = 0, available_1 = 0;
                }
                else {
                    response = result;
                }
            }
            else {
                response.push({ name: 'COGNITIVE_ANALYSIS', amount: 0, available: 0, retained: 0 });
                response.push({ name: 'SEMANTIC_ANALYSIS', amount: 0, available: 0, retained: 0 });
                response.push({ name: 'FREE', amount: 0, available: 0, retained: 0 });
                response.push({ name: 'TRANSCRIPTION', amount: 0, available: 0, retained: 0 });
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: { data: response } });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::/api/account/get_costs] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/save_account_currency', function (req, res, next) {
        var account = req.body.account || null;
        var newAccountBalance = req.body.accountBalance || null;
        var newAccountRates = req.body.accountRates || null;
        var newCountry = req.body.accountCountry || null;
        var newCurrency = req.body.accountCurrency || null;
        var changeValue = req.body.changeValue || 0;
        var convertAccountRates = req.body.convertRates || false;
        var convertSubAccountsRates = req.body.convertSubAccountsRates || false;
        var myUser = res.locals.userId || null;
        logger.info('[AccountService::save_account_currency] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (0 < newAccountRates.transcriptionRatePerHour
                &&
                    newAccountRates.transcriptionRatePerHour < 0.036)
                throw 'SERVER_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION';
            return mssql_service_1.MSSql.SaveAccountCurrency(account.installationId, account.accountId, newCountry.code, newCurrency.code, convertAccountRates, convertSubAccountsRates, newAccountRates.transcriptionRatePerHour, newAccountRates.transcriptionRatePerSecond, newAccountRates.semanticAnalysisRatePerTransaction, newAccountRates.cognitiveAnalysisRatePerTransaction, changeValue, myUser);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CURRENCY_UPDATE_FAILED';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::save_account_currency] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_movement_currencies', function (req, res, next) {
        var platformHash = req.body['platform-hash'] || req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var startDate = new Date(req.body.startDate) || null;
        var endDate = new Date(req.body.endDate) || null;
        var accountId = req.body.accountId || null;
        var subAccountId = req.body.subAccountId || null;
        var vcc = req.body.vcc || null;
        logger.info('[AccountService::get_movement_currencies] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetMovementCurrencies(installationId, startDate, endDate, accountId, subAccountId, vcc);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_movement_currencies] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var id = req.body.id || null;
        var accountId = req.body.account || null;
        var subAccountId = req.body.subAccountId || null;
        var platform = new platform_model_1.Platform;
        var resData = {
            account: null,
            accountBalance: null
        };
        logger.info('[AccountService::get_account2] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (platformHash && id) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[AccountService::get_account] Received params: ', platformHash, id);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_PLATFORM_NOT_FOUND';
            else if (res.headersSent)
                return;
            platform = result;
            return mssql_service_1.MSSql.GetAccount(platformHash, platform.installationId, id);
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_NOT_FOUND';
            else {
                resData.account = result;
                if (result.subAccountId)
                    return mssql_service_1.MSSql.GetAccountSubAccountBalance(platform.installationId, result.accountId, result.subAccountId, null, 'SCTA');
                else
                    return mssql_service_1.MSSql.GetAccountSubAccountBalance(platform.installationId, result.accountId, null, null, 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_BALANCE';
            else {
                if (res.headersSent)
                    return;
                resData.accountBalance = result;
                res.json({ status: true, description: 'OK', data: resData });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_account] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_accounts', function (req, res, next) {
        var installationId = req.body.installationId || null;
        logger.info('[AccountService::get_accounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetRelatedAccounts(installationId);
            }
            else {
                logger.info('[AccountService::get_accounts] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_subaccounts', function (req, res, next) {
        var installationId = req.body.installationId || null;
        logger.info('[AccountService::get_subaccounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetRelatedSubAccounts(installationId);
            }
            else {
                logger.info('[AccountService::get_subaccounts] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_subaccounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account_balance', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var accountId = req.body.account || null;
        var resData = {
            accountBalance: null,
            accountRate: null,
            subAccountsBalance: null,
            subAccountsRate: null
        };
        logger.info('[AccountService::get_account_balance] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId && accountId) {
                return mssql_service_1.MSSql.GetAccountSubAccountExists(installationId, accountId, null);
            }
            else {
                logger.info('[AccountService::get_account_balance] Received params: ', installationId, accountId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_NOT_FOUND';
            else {
                if (res.headersSent)
                    return;
                return mssql_service_1.MSSql.GetAccountSubAccountRate(installationId, accountId, null, 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_RATE';
            else {
                if (res.headersSent)
                    return;
                resData.accountRate = result[0];
                return mssql_service_1.MSSql.GetAccountSubAccountBalance(installationId, accountId, null, 'FREE', 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_BALANCE';
            else {
                if (res.headersSent)
                    return;
                resData.accountBalance = result[0];
                return mssql_service_1.MSSql.GetAccountSubAccountRate(installationId, accountId, null, 'SCTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT_RATES';
            else {
                if (res.headersSent)
                    return;
                resData.subAccountsRate = result;
                return mssql_service_1.MSSql.GetAccountSubAccountBalance(installationId, accountId, null, 'FREE', 'SCTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT_BALANCES';
            else {
                if (res.headersSent)
                    return;
                resData.subAccountsBalance = result;
                res.json({ status: true, description: 'OK', data: resData });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            if (errorMsg.split(' ').length === 1)
                clientMsg = errorMsg;
            logger.error('[AccountService::get_account_balance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AccountService = AccountService;
function GetMovements(installationId, startDate, endDate, accountId, subAccountId, vcc, currencies) {
    return mssql_service_1.MSSql.GetMovements(installationId, startDate, endDate, accountId, subAccountId, vcc, currencies.join(','));
}
function GetCosts(installationId, startDate, endDate, accountId, subAccountId, vcc) {
    return mssql_service_1.MSSql.GetCosts(installationId, startDate, endDate, accountId, subAccountId, vcc);
}
